package controls;

import pixi.core.display.Container;
import pixi.core.math.Point;
import pixi.core.sprites.Sprite;
import util.Asset;

/**
 * ...
 * @author Henri Sarasvirta
 */
class GameView extends Container
{
	public var grid:Grid;
	public var powerup:Sprite;
	public var powerpos:Point;
	
	private var controls:controls.Controllers;
	private var currentPiece:Piece;
	private var currentWorm:Worm;
	
	private var move:Int = 0;
	
	private var wormTickTime:Float = 200;
	private var tickTime:Float = 400;
	private var dropOn:Bool = false;
	private var startTime:Float = 0;
	private var wormStartTime:Float = 0;
	private var moveTime:Float = 0;
	
	private var tickStart:Float = 30;
	
	public function new() 
	{
		super();
		this.initializeControls();
	}
	
	private function initializeControls():Void
	{
		this.grid = new Grid();
		
		this.addChild(this.grid);
		
		this.powerup = Asset.getImage("powerup.png", true);
		this.powerup.width = this.powerup.height = Grid.SQUARE;
		
		spawnNext();
		spawnNextWorm();
		spawnNextPower();
		
		this.controls = new controls.Controllers();
		this.controls.addListener(Controllers.LEFT_ON, onLeft);
		this.controls.addListener(Controllers.RIGHT_ON, onRight);
		this.controls.addListener(Controllers.LEFT_OFF, offLeft);
		this.controls.addListener(Controllers.RIGHT_OFF, offRight);
		this.controls.addListener(Controllers.DROP_ON, onDrop);
		this.controls.addListener(Controllers.DROP_OFF, onDropOff);
		this.controls.addListener(Controllers.ROTATE, onRotate);
		
		
		this.controls.addListener(Controllers.WORM_LEFT, wormLeft);
		this.controls.addListener(Controllers.WORM_RIGHT, wormRight);
		this.controls.addListener(Controllers.WORM_UP, wormUp);
		this.controls.addListener(Controllers.WORM_DOWN, wormDown);
		
	}
	
	private function spawnNextPower():Void 
	{
		this.powerpos = new Point(
			Math.floor(Grid.WIDTH * Math.random()),
			Math.floor(Grid.HEIGHT * Math.random())
			
		);
		this.powerup.x = this.powerpos.x * Grid.SQUARE;
		this.powerup.y = this.powerpos.y * Grid.SQUARE;
		
		this.addChild(this.powerup);
	}
	
	private function wormLeft():Void
	{
		if (this.currentWorm.dirX == 0)
		{
			this.currentWorm.dirX = -1;
			this.currentWorm.dirY = 0;
		}
	}
	private function wormRight():Void
	{
		if (this.currentWorm.dirX == 0)
		{
			this.currentWorm.dirX = 1;
			this.currentWorm.dirY = 0;
		}
	}
	
	private function wormDown():Void
	{
		if (this.currentWorm.dirY == 0)
		{
			this.currentWorm.dirY = 1;
			this.currentWorm.dirX = 0;
		}
	}
	
	private function wormUp():Void
	{
		if (this.currentWorm.dirY == 0)
		{
			this.currentWorm.dirY = -1;
			this.currentWorm.dirX = 0;
		}
	}
	
	private function spawnNext():Void
	{
		this.currentPiece = new Piece();
		this.currentPiece.gx = Math.floor((Grid.WIDTH - this.currentPiece.gridWidth) / 2);
		this.currentPiece.x = this.currentPiece.gx * Grid.SQUARE;
		this.currentPiece.y = this.currentPiece.gy * Grid.SQUARE;
		this.addChild(this.currentPiece);
		
		if (this.grid.hits(this.currentPiece))
		{
			//game over.
			this.grid.reset();
		}
	}
	private function spawnNextWorm():Void
	{
		this.currentWorm = new Worm();
		this.addChild(this.currentWorm);
	}
	
	private function offLeft():Void
	{
		move = 0;
	}
	private function offRight():Void
	{
		move = 0;
	}
	
	private function onLeft():Void
	{
		if (move == -1) return;
		move = -1;
		moveLeft();
	}
	private function moveLeft():Void
	{
		if (this.currentPiece.gx > 0)
		{
			this.currentPiece.gx--;
			if (this.grid.hits(this.currentPiece))
				this.currentPiece.gx++;
			this.currentPiece.x = this.currentPiece.gx * Grid.SQUARE;
		}
		this.moveTime = Date.now().getTime();
		
	}
	
	private function onRight():Void
	{
		if (move == 1) return;
		move = 1;
		moveRight();
	}
	
	private function moveRight():Void
	{
		if (this.currentPiece.gx < Grid.WIDTH-1 - this.currentPiece.gridWidth)
		{
			this.currentPiece.gx++;
			if (this.grid.hits(this.currentPiece))
				this.currentPiece.gx--;
			this.currentPiece.x = this.currentPiece.gx * Grid.SQUARE;
		}
		this.moveTime = Date.now().getTime();
	}
	
	private function onDrop():Void
	{
		this.dropOn = true;
	}
	private function onDropOff():Void
	{
		this.dropOn = false;
	}
	
	private function onRotate():Void
	{
		
		this.currentPiece.rotate();
		while (this.currentPiece.gx + this.currentPiece.gridWidth >= Grid.WIDTH)
			this.currentPiece.gx--;
		while (this.currentPiece.gx < 0)
			this.currentPiece.gx++;
		this.currentPiece.x = this.currentPiece.gx * Grid.SQUARE;
		this.currentPiece.y = this.currentPiece.gy * Grid.SQUARE;
		if (this.grid.hits(this.currentPiece))
		{
			//Rotate back
			this.currentPiece.rotate();
			this.currentPiece.rotate();
			this.currentPiece.rotate();
		}
		
	}
	
	public function onTick(delta:Float):Void
	{
		var now:Float = Date.now().getTime();
		
		
		
		if (now - moveTime > 100)
		{
			if (move == -1) moveLeft();
			else if (move == 1) moveRight();
		}
		
		if (now - wormStartTime > wormTickTime)
		{
			wormStartTime = now;
			//Move the worm.
			this.currentWorm.moveWorm();
			if (this.grid.wormHits(this.currentWorm, this.currentPiece))
			{
				this.currentWorm.revertPieces();
			//	this.grid.dropWorm(this.currentWorm);
				this.removeChild(this.currentWorm);
				this.spawnNextWorm();
			}
			else if (this.currentWorm.coordinates[0].x == powerpos.x && this.currentWorm.coordinates[0].y == powerpos.y)
			{
				this.spawnNextPower();
				this.currentWorm.increaseSize();
			}
		}
		if (now - tickStart > 30)
		{
			grid.tick(this.currentPiece);
			tickStart = now;
		}
		
		if (now - startTime > tickTime || dropOn && now-startTime > 20)
		{
			startTime = now;
			grid.tick(this.currentPiece);
			this.currentPiece.gy++;
			
			//test hit
			if (this.grid.hits(this.currentPiece))
			{
				this.currentPiece.gy--;
				this.dropOn = false;
				this.grid.populate(this.currentPiece);
				this.removeChild(this.currentPiece);
				spawnNext();
			}
			
			
			this.currentPiece.y = this.currentPiece.gy * Grid.SQUARE;
			
			
		}
	}
}