package controls;

import createjs.easeljs.Rectangle;
import pixi.core.display.Container;
import pixi.core.graphics.Graphics;
import pixi.core.graphics.GraphicsData;
import pixi.core.math.Point;
import pixi.core.sprites.Sprite;
import util.IntStepIter;

/**
 * ...
 * @author Henri Sarasvirta
 */
class Grid extends Container
{
	public static var WIDTH:Int = 12;
	public static var HEIGHT:Int= 25;
	
	public static var SQUARE:Int = 30;
	
	public var grid:Array<Array<GridItem>>;
	
	private var bg:Graphics;
	private var items:Container;
	
	private var size:Rectangle;
	
	public function new() 
	{
		super();
		this.initializeControls();
	}
	
	public function reset():Void
	{
		for ( i in 0...HEIGHT)
		{
			for (j in 0...WIDTH)
			{
				var c:GridItem = grid[i][j];
				if (c.sprite != null)
					this.removeChild(c.sprite);
				c.sprite = null;
			}
		}
	}
	
	public function resize(size:Rectangle):Void
	{
		this.size = size;
		//Scale the stage.
	}
	
	private function initializeControls():Void
	{
		
		this.bg = new Graphics();
		this.bg.beginFill(0xFFFFFF);
		this.bg.drawRect(0, 0, SQUARE*WIDTH, SQUARE*HEIGHT);
		this.bg.endFill();
		
		this.addChild(this.bg);
		
		this.grid = [];
		for ( i in 0...HEIGHT)
		{
			this.grid[i] = [];
			for (j in 0...WIDTH)
			{
				grid[i][j] = new GridItem();
			}
		}
		
	}
	
	
	public function dropPiece(piece:Piece):Void
	{
		//find out how far the piece can drop
		var yStart:Int = piece.gy;
		for (y in yStart...HEIGHT)
		{
			piece.gy = y;
			if (hits(piece))
			{
				//Stop movement.
				piece.gy--;
				return;
			}
		}
	}
	
	public function populate(piece:Piece)
	{
		for (s in piece.squares)
		{
			s.x = piece.gx * SQUARE + s.x;
			s.y = piece.gy * SQUARE + s.y;
			
			this.addChild(s);
		}
		
		for (i in 0...piece.coordinates.length)
		{
			var coordinate:Point = piece.coordinates[i];
			var s:Sprite = piece.squares[i];
			this.grid[cast coordinate.y+piece.gy][cast coordinate.x + piece.gx].sprite = s;
		}
		
		//Check if row is full.
		var remove:Array<Int> = [];
		for (i in 0...HEIGHT)
		{
			var full:Bool = true;
			for (j in 0...WIDTH)
			{
				full = grid[i][j].sprite != null;
				if (!full)
					break;
			}
			if (full)
			{
				remove.push(i);
			}
		}
		for (ind in remove)
		{
			for (j in 0...WIDTH)
			{
				this.removeChild(grid[ind][j].sprite);
				grid[ind][j].sprite = null;
				grid[ind][j].isWorm = false;
			}
			
			for (i in new IntStepIter(ind, 1, -1))
			{
				for (j in 0...WIDTH)
				{
					if(grid[i-1][j].sprite != null)
						grid[i - 1][j].sprite.y += SQUARE;
					grid[i][j].sprite = grid[i - 1][j].sprite;
					grid[i][j].isWorm = grid[i - 1][j].isWorm;
					
				}
			}
		}
	}
	
	public function hits(piece:Piece):Bool
	{
		for (coordinate in piece.coordinates)
		{
			var cx:Int = piece.gx + cast coordinate.x;
			var cy:Int = piece.gy + cast coordinate.y;
			
			if (cy >= HEIGHT)
			{
				trace( piece.gridHeight);
				//hits the bottom floor.
				return true;
			}
			
			if (this.grid[cy][cx].sprite != null)
			{
				//Hit found
				return true;
			}
		}
		
		return false;
	}
	
	public function wormHits(worm:Worm, currentPiece:Piece):Bool
	{
		
		var same:Array<Int> = [];
		for (point in worm.coordinates)
		{
			//check self
/*			var ind:Int =cast point.y * WIDTH + x;
			if (same.indexOf(ind) >= 0)
				return true;
			same.push(ind);
	*/		
			//check static
			if (point.x >= WIDTH || point.x < 0 || point.y < 0 || point.y >= HEIGHT || this.grid[cast point.y][cast point.x].sprite != null)
			{
				return true;
			}
			
			//check currentpiece
			for (cp in currentPiece.coordinates)
			{
				if (cp.x + currentPiece.gx == point.x && cp.y + currentPiece.gy == point.y)
					return true;
			}
		}
		
		
		return false;
	}
	
	public function dropWorm(worm:Worm):Void
	{
		for (i in 0...worm.pieces.length)
		{
			var c:Point = worm.coordinates[i];
			var s:Sprite = worm.pieces[i];
			this.grid[cast c.y][cast c.x].sprite = s;
			this.grid[cast c.y][cast c.x].isWorm = true;
			this.addChild(s);
		}
	}
	
	public function tick(currentPiece:Piece):Void
	{
		//Drop worms if possible
		for ( i in new IntStepIter(HEIGHT-1, -1,-1))
		{
			for (j in 0...WIDTH)
			{
				if (grid[i][j].isWorm && grid[i][j].sprite != null)
				{
					if (i < HEIGHT - 1 && grid[i+1][j].sprite == null && !currentPiece.testHit(i+j,j))
					{
						grid[i + 1][j].sprite = grid[i][j].sprite;
						grid[i][j].sprite = null;
						grid[i][j].isWorm = false;
						grid[i + 1][j].sprite.y += SQUARE;
						grid[i + 1][j].isWorm = true;
					}
				}
			}
		}
	}
}