package;
import controls.GameView;
import controls.StartView;
import createjs.easeljs.Ticker;
import haxe.Timer;
import js.Browser;
import js.html.CanvasElement;
import js.html.DivElement;
import js.html.Element;
import js.JQuery;
import js.Lib;
import pixi.core.display.Container;
import pixi.core.math.shapes.Rectangle;
import pixi.core.renderers.Detector;
import pixi.core.renderers.Detector.RenderingOptions;
import pixi.core.renderers.SystemRenderer;
import pixi.loaders.Loader;
import sounds.Sounds;
import util.Asset;



/**
*/
@:expose("Main")
class Main
{
	public static var instance:Main;
	
	
	private var previousTick:Float;
	/**
	 * Reference to main container. Used directly by classes requiring dom access.
	 */
	public var container:DivElement;
	
	/**
	* Private properties of the class.
	*/
	private var resizeTimer:Timer;
	private var mainCanvas:CanvasElement;
	public var mainStage:Container;
	public var renderer:SystemRenderer;
	
	private var loader:Loader;
	
	private var game:GameView;
	
	private var start:StartView;
	
	/**
	* Entry point of the application.
	*/
	static public function main():Void
	{
		
	}
	
	/**
	* Creates an new instance of the class.
	*/
	public function new():Void
	{
		instance = this;
		configure();
	}
	
	/**
	* The game host (framework) tells to configure the game.
	* @param framework The reference to the framework.
	*/
	public function configure():Void
	{	
		Sounds.soundsLoadedHandler = onSoundsLoaded;
		Sounds.initSounds();
		
		
		this.loader = new Loader();
		var assets:Array<String> = Config.GAME_ASSETS;
		
		//Instead of frameworks loader wrap pixi.js loader to show loading progress in fw loader.
		var assetCount:Int = 0;
		for (i in 0...assets.length)
		{
			loader.add(assets[i], Config.GAME_PATH+assets[i]);
		}
		loader.addListener("complete", function() {
			this.allAssetsLoaded();
		});
		Asset.init(this, this.loader);
		
	}
	private function onSoundsLoaded():Void
	{
		loader.load();
		
	}
	
	public function getGameSize():Rectangle
	{
		var width:Float = Browser.window.innerWidth;
		var height:Float = Browser.window.innerHeight;
		return new Rectangle(0, 0, width, height);
	}
	
	private function allAssetsLoaded():Void
	{
		this.container = cast Browser.document.getElementById("container");
		this.initializeControls(container);
		Browser.window.addEventListener("resize", this.onResize, false);
	
	}
	
	/**
	* After a resize event, throttle it and then resize the required elements.
	* @param event The recieved resize event.
	*/
	private function onResize(event:Dynamic):Void
	{
		if (this.resizeTimer != null) this.resizeTimer.stop();
		this.resizeTimer = Timer.delay(function() {
			var size:Rectangle = this.getGameSize();
			this.renderer.resize(size.width, size.height);
			var s:Float = size.height / 720;
			game.scale.x = game.scale.y = s;
			game.x = Math.round((size.width - game.width) / 2);
			start.resize(size);
		
		},
		50);
	}
	
	/**
	* Initializes the main controls used in the game UI.
	* @param container The container where to add the canvas.
	*/
	private function initializeControls(container:DivElement):Void
	{
		/**
		* gameSize
		*/
		var size:Rectangle = this.getGameSize();
		var gameWidth:Float = size.width;
		var gameHeight:Float = size.height;
		
		//Mainstage build
		this.mainStage = new Container();
		
		//Renderer
		var rendererOptions:RenderingOptions = { };
		rendererOptions.backgroundColor = 0xff9090;
		rendererOptions.antialias = false;
		this.renderer = Detector.autoDetectRenderer(gameWidth, gameHeight, rendererOptions);
		container.appendChild(this.renderer.view);
		
		this.game = new GameView();
		
		this.game.visible = false;
		
		this.start = new StartView();
		
		this.start.addListener(StartView.TYPE_SELECTED, moveToGame);
		
		this.mainStage.interactiveChildren = true;
		
		this.mainStage.addChild(this.game);
		this.mainStage.addChild(this.start);
		
		/*
		* Ticker
		*/
		Ticker.addEventListener("tick", tick);
		
		onResize(null);
		
		this.start.prepareView();
		
	}
	
	private function moveToGame():Void
	{
		this.game.prepare();
		this.start.visible = false;
		this.game.visible = true;
	}
	
	private function backToMenu():Void
	{
		this.start.prepareView();
	}
	
	/**
	* Updates the stage on Ticker tick.
	* @param event The recieved tick event.
	*/
	private function tick(event:Dynamic):Void
	{
		
		//Always update as there's always something moving.
		var now:Float = Date.now().getTime();
		var delta:Float = now - previousTick;
		previousTick = now;
		//Allow maximum of 0.2s. delta
		delta = Math.min(200, delta);
		this.renderer.render(this.mainStage);
		//frametime should be 16.6 ms. Calculate multiplier offset to effects.
		delta /= 16.666;
		
	}
}
