package controls;

import createjs.tweenjs.Ease;
import createjs.tweenjs.Tween;
import pixi.core.display.Container;
import pixi.core.graphics.Graphics;
import pixi.core.math.Point;
import pixi.core.math.shapes.Rectangle;
import pixi.core.sprites.Sprite;
import pixi.extras.TilingSprite;
import util.Asset;

/**
 * ...
 * @author Henri Sarasvirta
 */
class Card extends Container
{
	public var discarded:Bool = false;
	
	private var back:Sprite;
	private var front:Sprite;
	
	public var data:Dynamic;
	public var basetype:String;
	
	private var ox:Int;
	private var oy:Int;
	
	public var isMouseover:Bool = false;
	
	public var hitClip:Graphics;
	
	public var attachedCards:Array<Card>;
	public var baseValue:Int;
	public var slotAmount:Int;
	
	public function new(json:Dynamic, basetype:String) 
	{
		super();
		this.scale.x = this.scale.y = 2;
		this.data = json;
		this.basetype = basetype;
		attachedCards = new Array<Card>();
		slotAmount = Std.parseInt( json.slots);
		baseValue =json.price | json.skill | 0;
		this.initializeControls();
	}
	
	public function whichHand():String
	{
		return this.basetype == "hahmot" || this.basetype == "autot" ? "line" : "hand";
	}
	
	public function showCard():Void
	{
		if(this.front != null)
		this.front.visible = true;
		this.back.visible = false;
		
		for (i in 0...attachedCards.length) {
			attachedCards[i].showCard();
		}
	}
	public function isAnEvent() {
		return data.name == "vihje" || data.name == "kissanristiaiset" || data.name == "pikkujoulut" || data.name=="virka_apu" || data.name=="out_of_order" || data.name=="haat";
	}
	
	public function calculateValue(onlyRevealed:Bool):Int {
		var value = baseValue;
		if(!onlyRevealed || (onlyRevealed && this.front.visible)){
			
			for (i in 0...attachedCards.length) {
				value += attachedCards[i].calculateValue(onlyRevealed);
			}
			
			return value;
		}
		return 0;
	}
	public function isContraband():Bool {
		return data.type == "hard_drug";
	}
	public function mutatorMultiplier():Int {
		return basetype == "mutator" ? data.value : 1;
	}
	public function hasRevealedContraband():Bool {
		var revealedC = this.front.visible && isContraband();
		for (i in 0...attachedCards.length) {
			revealedC = revealedC || attachedCards[i].hasRevealedContraband();
		}
	return revealedC;
	}

	
	public function calculateGiftValues():Int {
		var value = data.type == "gift" ? data.price : 0;
		for (i in 0...attachedCards.length) {
			value += attachedCards[i].calculateGiftValues();
			
 		}
		return value;
		this.hitClip.addListener("mouseover", onMouseOver);
		this.hitClip.addListener("mouseout", onMouseOut);
		
 	}

	public function attachCard(card:Card) {
		var op:Point = card.toGlobal(new Point(0, 0));
		op = this.toLocal(op);
		Tween.get(card.pivot).to( { x:0, y:0 }, 250, Ease.quadInOut);
		Tween.get(card).to( { x:this.attachedCards.length*15, y: -20 }, 250, Ease.quadInOut);
		Tween.get(card.scale).to( { x:0.5, y:0.5 }, 250, Ease.quadInOut);
		
		//TODO: position
		this.addChild(card);
		this.attachedCards.push(card);
	}
 	
	private function initializeControls():Void
	{
		//Back calculation
		if (basetype == "mutators")
		{
			//event / mutator cards
			back = Asset.getImage("Tapahtumat.png",true);
		}
		else if (basetype == "hahmot")
		{
			//officer card
			back = Asset.getImage("Hahmot.png",true);
		}
		else if (basetype == "autot")
		{
			//smuggler cars
			back = Asset.getImage("Autot.png",true);
		}
		else if (basetype == "kamat")
		{
			//smuggler use cards
			back = Asset.getImage("Kamat.png",true);
		}
		else if (basetype == "tullikamat")
		{
			//officer use cards
			back = Asset.getImage("TulliKamat.png",true);
		}
		else if (basetype == "tullirajat")
		{
			//import limits per round
			back = Asset.getImage("TulliRajat.png",true);
		}
		
		this.addChild(this.back);
		
		var ha:Rectangle = new Rectangle(0, 0, this.back.width, this.back.height);
		untyped this.hitArea = ha;
		this.hitClip = new Graphics();
		this.hitClip.beginFill(0x0, 0);
		this.hitClip.drawRect(0, 0, this.back.width*2, this.back.height*2);
		this.hitClip.endFill();
		this.hitClip.buttonMode = true;
		
		//Front
		try {
		trace(data.image);	
		this.front = Asset.getImage(data.image, true);
		this.front.visible = false;
		this.addChild(this.front);

		}
		catch (e:Dynamic) {
		trace(basetype);
		trace(data.image);
			
		}
		this.hitClip.addListener("mouseover", onMouseOver);
		this.hitClip.addListener("mouseout", onMouseOut);
		
	}
	
	private function onMouseOver():Void
	{
		Tween.removeTweens(this.pivot);
		Tween.removeTweens(this.scale);
		
		trace("over");
		this.isMouseover = true;
		this.scale.x = this.scale.y = 2;
		Tween.get(this.pivot).to( { x:this.front.width/8, y:this.front.height/5 }, 150, Ease.backOut);
		Tween.get(this.scale).to( { x:2.5, y:2.5 }, 150, Ease.backOut);
		
		untyped this.parent.updateOrder();
	}
	
	private function onMouseOut():Void
	{
		this.isMouseover = false;
		Tween.removeTweens(this.pivot);
		Tween.removeTweens(this.scale);
		Tween.get(this.pivot).to( { x:0, y:0 }, 150, Ease.backIn);
		Tween.get(this.scale).to( { x:2, y:2 }, 150, Ease.backIn);
		untyped this.parent.updateOrder();
	}
}