package controls;

import createjs.tweenjs.Tween;
import haxe.Timer;
import js.Lib;
import pixi.core.display.Container;
import pixi.core.math.Point;
import pixi.core.math.shapes.Rectangle;
import pixi.core.sprites.Sprite;
import pixi.interaction.InteractionData;
import sounds.Sounds;
import util.Asset;

/**
 * This implements the basic game view.
 * @author Henri Sarasvirta
 */
class GameView extends Container
{
	private var controls:controls.Controllers;
	private var smuggler:SideBaseView;
	private var customs:SideBaseView;
	public var lines:Array<Line>;
	private var bgLayer:Container;
	
	private var player:Player;
	private var enemy:Enemy;
	
	private var gameDeck:CardPack;
	private var state:GameStateMachine;
	
	private var turnDone:Sprite;
	
	public function new()
	{
		super();
		this.initializeControls();
	}
	
	private function initializeControls():Void
	{
		this.gameDeck = new CardPack();
		this.state = new GameStateMachine();
		this.controls = new controls.Controllers();
		this.lines = new Array<Line>();
		
		this.bgLayer = new Container();
		
		this.smuggler = new SideBaseView("smuggler");
		this.customs = new SideBaseView("customs");
		
		this.turnDone = Asset.getImage("Valmis.png", true);
		this.turnDone.x = 350;
		this.turnDone.y = 550;
		this.turnDone.interactive = true;
		this.turnDone.buttonMode = true;
		this.turnDone.addListener("click", onTurnDoneClick);
		this.turnDone.addListener("tap", onTurnDoneClick);
		
		this.enemy = new Enemy();
		this.player = new Player();
		
		this.addChild(this.bgLayer);
		for (i in 0...3) {
			CreateLine(i);
		}
		this.addChild(this.customs);
		this.bgLayer.addChild(this.customs.bg);
		
		this.addChild(this.smuggler);
		this.bgLayer.addChild(this.smuggler.bg);
		
		this.addChild(this.turnDone);
	}
	
	private function prepareSmugglerDecks() {
		this.smuggler.lineDeck.prepare(this.gameDeck.getTransportDeck());
		this.smuggler.handDeck.prepare(this.gameDeck.getSmugglerHandDeck());
	}
	
	private function prepareCustomsDecks() {		
		this.customs.lineDeck.prepare(this.gameDeck.getCustomsOfficerDeck());
		this.customs.handDeck.prepare(this.gameDeck.getCustomsHandDeck());
	}
	
	private function prepareCommonDecks() {
		//this.smuggler.limitDeck.prepare(this.gameDeck.getLimitDeck());
		this.smuggler.eventDeck.prepare(this.gameDeck.getMutatorDeck());
	}
	
	private function CreateLine(number:Int):Void {
		var line = new Line();
		line.x = 340 + number * 200;//line.getBounds().width;
		line.y = 180;
		this.lines.push(line);
		this.addChild(line);
	}	
	
	public function prepare():Void
	{
		this.turnDone.visible = false;
		var si = Sounds.playEffect(Sounds.BACKGROUND, -1, 0, 300);
		Tween.get(si).to( { "volume":1 }, 500);
		
		if (StartView.GAME_TYPE == StartView.GAME_TYPE_SMUGGLER)
		{
			this.smuggler.y = 720 / 2;
			this.customs.y = 0;
			this.customs.bg.scale.y = -1;
			this.customs.bg.y = 720 / 2;
			this.smuggler.bg.scale.y = 1;
			this.smuggler.bg.y = 720/2;
			this.player.prepare(this.smuggler, this);
			this.enemy.prepare(this.customs, this);
			this.smuggler.interactive = true;
			this.customs.interactive = false;
			this.smuggler.addListener("mousedown", onCardMouseDown);
			this.customs.removeListener("mousedown", onCardMouseDown);
			this.smuggler.addListener("touchstart", onCardMouseDown);
			this.customs.removeListener("touchstart", onCardMouseDown);
			
		}
		else if (StartView.GAME_TYPE == StartView.GAME_TYPE_CUSTOMS)
		{
			this.customs.bg.scale.y = 1;
			this.customs.y = 720 / 2;
			this.smuggler.y = 0;
			this.smuggler.bg.scale.y = -1;
			this.smuggler.bg.y = 720 / 2;
			this.customs.bg.y = 720 / 2;
			
			this.player.prepare(this.customs, this);
			this.enemy.prepare(this.smuggler,this);
			this.smuggler.interactive = false;
			this.customs.interactive = true;
			
			this.customs.addListener("mousedown", onCardMouseDown);
			this.smuggler.removeListener("mousedown", onCardMouseDown);
			this.customs.addListener("touchstart", onCardMouseDown);
			this.smuggler.removeListener("touchstart", onCardMouseDown);
			
		}
		
		this.smuggler.prepare(StartView.GAME_TYPE == StartView.GAME_TYPE_CUSTOMS);
		this.customs.prepare(StartView.GAME_TYPE == StartView.GAME_TYPE_SMUGGLER);
		
		this.smuggler.interactiveChildren = true;
		this.customs.interactiveChildren = true;
		
		prepareSmugglerDecks();
		prepareCustomsDecks();
		prepareCommonDecks();
		
		this.interactive = true;
		
		state.NextState();
		advanceState();
	}
	
	private var pickUp:Point;
	private var cardDrag:Card;
	private var cardPickStage:Point;
	private function onCardMouseDown(e:Dynamic):Void
	{
		trace("md down");
		
		this.cardDrag = this.player.view.lineHand.getCardInPosition();
		if (this.cardDrag == null)
			this.cardDrag = this.player.view.hand.getCardInPosition();
		if (this.cardDrag != null)
		{
			this.player.view.interactiveChildren = false;
			var glob:Point = e.data.getLocalPosition(this);
			this.pickUp = e.data.getLocalPosition(this.cardDrag.parent);
			this.pickUp.x -= this.cardDrag.x;
			this.pickUp.y -= this.cardDrag.y;
			
			this.cardPickStage = e.data.getLocalPosition(this);
			this.cardDrag.x = glob.x - this.pickUp.x;
			this.cardDrag.y = glob.y - this.pickUp.y;
			this.addChild(this.cardDrag);
			Tween.removeTweens(this.cardDrag.pivot);
			Tween.removeTweens(this.cardDrag.scale);
			var hand:Hand = this.cardDrag.whichHand() == "hand" ? this.player.view.hand :
				this.player.view.lineHand;
			hand.removeCard(cardDrag);
			this.addListener("mouseup", onCardMouseUp);
			this.addListener("mousemove", onCardMouseMove);
			this.addListener("touchend", onCardMouseUp);
			this.addListener("touchmove", onCardMouseMove);
		}
	}
	private function onCardMouseMove(event:Dynamic):Void
	{
		var p:Point = event.data.getLocalPosition(this);
		trace("move");
		cardDrag.x = p.x - this.pickUp.x;
		cardDrag.y = p.y - this.pickUp.y;
		
	}
	
	private function boxHitTest(x1:Float, y1:Float, w1:Float, h1:Float, x2:Float, y2:Float, w2:Float, h2:Float):Bool
	{
		return x1 + w1 > x2 && 
		       x1 < x2 + w2 && 
			   y1 + h1 > y2 && 
			   y1 < y2 + h2;
	};
	
	private function onCardMouseUp(event:Dynamic):Void
	{
		this.player.view.interactiveChildren = true;
		var p:Point = event.data.getLocalPosition(this);
		var lhit:Line = null;
		var cb:Rectangle = this.cardDrag.getBounds();
		
		var findEmpty:Bool = this.cardDrag.basetype == "autot" || this.cardDrag.basetype == "hahmot";
		
		for (l in lines)
		{
			var lb:Rectangle = l.getBounds();
			if (!l.hasPlayerCard && boxHitTest(cb.x, cb.y, cb.width, cb.height, lb.x, lb.y, lb.width, lb.height))
			{
				lhit = l;
				break;
			}
			else if (!findEmpty && l.hasPlayerCard && boxHitTest(cb.x, cb.y, cb.width, cb.height, lb.x, lb.y, lb.width, lb.height))
			{
				lhit = l;
			}
		}
		
		this.removeListener("mouseup", onCardMouseUp);
		this.removeListener("mousemove", onCardMouseMove);	
		this.removeListener("touchend", onCardMouseUp);
		this.removeListener("touchmove", onCardMouseMove);	
		
		if (lhit != null)
		{
			//Set to line
			this.player.view.lineHand.resize();
			this.player.view.hand.resize();
			trace("LHIT");
			trace(lhit);
			
			if ( findEmpty)
			{
				if (StartView.GAME_TYPE == StartView.GAME_TYPE_SMUGGLER)
				{
					lhit.setTransport(cardDrag,0);
				}
				else
				{
					lhit.setCustomsOfficer(cardDrag,0);
				}
				
				if (this.player.view.lineHand.cards.length == 0)
				{
					this.turnDone.visible = true;
				}
			}
			else
			{
				if(lhit.playerCard.slotAmount > lhit.playerCard.attachedCards.length)
					lhit.playerCard.attachCard(this.cardDrag);
				else
				{
					//Return to hand
					var hand:Hand = this.cardDrag.whichHand() == "hand" ? this.player.view.hand:
						this.player.view.lineHand;
					hand.putCard(cardDrag);					
				}
			}
		}
		else
		{
			//Return to hand
			var hand:Hand = this.cardDrag.whichHand() == "hand" ? this.player.view.hand:
				this.player.view.lineHand;
			hand.putCard(cardDrag);
		}
	}
	
	private function onTurnDoneClick():Void
	{
		
		if (state.currentPosition == GameStateMachine.PlayImmediateCards)
		{
			this.interactiveChildren = false;
			this.turnDone.visible = false;
			this.state.NextState();
			this.advanceState();
		}
		else
		{
			this.interactiveChildren = false;
			this.turnDone.visible = false;
			this.state.NextState();
			this.advanceState();

		}
		
	}
	
	public function advanceState():Void
	{
		trace("advance to " +state.currentPosition);
		
		switch(state.currentPosition) {
			case GameStateMachine.DealSmugglerCards:
				HandleDealSmugglerCards();
			case GameStateMachine.DealCustomsCards:
				HandleDealCustomsCards();
			case GameStateMachine.ShowLineEvents:
				HandleShowLineEvents();
			case GameStateMachine.SmugglerPlaysCards:
				HandleSmugglerPlaysCards();
			case GameStateMachine.CustomsOfficerPlaysCards:
				HandleCustomsOfficerPlaysCards();
			case GameStateMachine.ResolveTurn:
				HandleResolveTurn();
			case GameStateMachine.PlayImmediateCards:
				HandlePlayImmediateCards();
			case GameStateMachine.ResolveFinal:
				HandleResolveFinal();
			case GameStateMachine.CleanUp:
				handleCleanUp();
			case GameStateMachine.endState:
				trace("THE END");
				Lib.alert("PELI LOPPUI! " + (player.view.points > enemy.view.points ? " VOITIT PELIN!" : "HÄVISIT PELIN"));
			default:
				//break;			
		}
	}
	
	private function moveToNextState():Void
	{
		this.state.NextState();
		this.advanceState();
	}
	
	//1.
	public function HandleDealSmugglerCards() {
		
		this.enemy.resetTurn();
		// start animation etc.. After animation call state machine next
		this.interactiveChildren = false;
		var delayMp:Float = StartView.GAME_TYPE == StartView.GAME_TYPE_SMUGGLER?1 :0.1 ;
		while (this.smuggler.hand.cards.length < 7)
		{	
			if (this.smuggler.handDeck.cards.length == 0) {
				// Shuffle new deck from discarded
				this.smuggler.handDeck.prepare(this.gameDeck.shuffleDeck(this.smuggler.handDeck.getDiscards()));
			}
			trace("card");
			var card = this.smuggler.handDeck.takeCard();
			this.smuggler.hand.putCard(card,cast this.smuggler.hand.cards.length*250*delayMp);
		}
		while (this.smuggler.lineHand.cards.length < 3)
		{	
			/*
			if (this.smuggler.lineDeck.cards.length == 0) {
				// Shuffle new deck from discarded
				this.smuggler.lineDeck.prepare(this.gameDeck.shuffleDeck(this.smuggler.lineDeck.getDiscards()));
			}*/
			var card = this.smuggler.lineDeck.takeCard();
			this.smuggler.lineHand.putCard(card, cast (7 * 250 + this.smuggler.lineHand.cards.length * 250)*delayMp);/// .call( this.smuggler.lineHand.cards.length == 3 ? moveToNextState : null);
		}
		
		//TODO - this should be handled via event throuhg putcard animationg
		Timer.delay(moveToNextState,cast 10 * 250*delayMp);
		
		
	}
	
	
	//2.
	public function HandleDealCustomsCards() {
		
		var delayMp:Float = StartView.GAME_TYPE == StartView.GAME_TYPE_SMUGGLER?0.1 :1 ;
		
		this.interactiveChildren = false;
		while (this.customs.hand.cards.length < 7)
		{
			trace("added card");
			if (this.customs.handDeck.cards.length == 0) {
				// Shuffle new deck from discarded
				this.customs.handDeck.prepare(this.gameDeck.shuffleDeck(this.customs.handDeck.getDiscards()));
			}
			var card = this.customs.handDeck.takeCard();
			this.customs.hand.putCard(card,cast this.customs.hand.cards.length*250*delayMp);
		}
		
		while (this.customs.lineHand.cards.length < 3)
		{	
			trace("added card");
			if (this.customs.lineDeck.cards.length == 0) {
				// Shuffle new deck from discarded
				this.customs.lineDeck.prepare(this.gameDeck.shuffleDeck(this.customs.lineDeck.getDiscards()));
			}
			var card = this.customs.lineDeck.takeCard();
			this.customs.lineHand.putCard(card,cast (7 * 250 + this.customs.lineHand.cards.length * 250)*delayMp);
		}
		
		Timer.delay(moveToNextState,cast 10 * 250*delayMp);
		
	}
	
	//3.
	public function HandleShowLineEvents() {
		trace("handle show events");
		this.interactiveChildren = false;
		//currentPosition = SmugglerPlaysCards;
		// start animation etc.. After animation currenPosition++
		for (i in 0...lines.length) {
			if (this.smuggler.eventDeck.cards.length == 0) {
				this.smuggler.eventDeck.prepare(this.gameDeck.shuffleDeck(this.smuggler.eventDeck.getDiscards()));
			}
			var mutator = smuggler.eventDeck.takeCard();
			lines[i].setMutator(mutator,i*150+10);
		}
		this.state.NextState();
		Timer.delay(advanceState, 400 );
	}
	
	//4.
	public function HandleSmugglerPlaysCards() {
		// .. After animation currenPosition++
		//currentPosition = ShowLineEvents;
		Sounds.playEffect(Sounds.LUCAS_EI_TULLATTAVAA,0,3);
		if (StartView.GAME_TYPE == StartView.GAME_TYPE_CUSTOMS) {
			this.interactiveChildren = false;
			this.enemy.executeTurn();
			if (this.enemy.turnDone) {
				this.state.NextState();
				Timer.delay(advanceState, 400 );
				
			}
		}
		else {
			// player plays	
			this.interactiveChildren = true;
		}
	}
	
	//5.
	public function HandleCustomsOfficerPlaysCards() {
		// start animation etc.. After animation currenPosition++
		//currentPosition = CustomsOfficerPlaysCards;
		if(StartView.GAME_TYPE == StartView.GAME_TYPE_SMUGGLER){
			this.enemy.executeTurn();
			if (this.enemy.turnDone) {
				this.state.NextState();
				Timer.delay(advanceState, 400 );
			}
		}
		else {
			// player plays	
			this.interactiveChildren = true;
		}
	}
	
	//6.
	public function HandleResolveTurn() {
		trace("RESOLVE");
		//Animate cards to view.
		if (state.resolveStepPosition < 3) {
			var l = lines[state.resolveStepPosition];
			var transport = l.getTransport();
			var officer = l.getOfficer();
			var mutator = l.getMutator();
			
			l.customsText.text = cast officer.calculateValue(false);
			l.smugglerText.text = cast transport.calculateValue(false);
			
			transport.showCard();
			officer.showCard();
			mutator.showCard();
			state.ResolveNextLine();
			
			if (state.resolveStepPosition == 1){
				switch(officer.data.name) {
				case "kake":
					Sounds.playEffect(Sounds.JERE_SEIS,0,3);
				case "pertsa":
					Sounds.playEffect(Sounds.JERE_SEIS,0,3);
				case "rane":
					Sounds.playEffect(Sounds.JERE_SEIS,0,3);
				case "mä":
					Sounds.playEffect(Sounds.JERE_SEIS,0,3);
				case "marjatta":
					Sounds.playEffect(Sounds.HANNA_SEIS,0,3);
				case "pirita":
					Sounds.playEffect(Sounds.HANNA2_SEIS,0,3);
				case "helga":
					Sounds.playEffect(Sounds.JERE2_SYVA_KYYKKY,0,3);
				default:
					//break;			
				}
			}
			Timer.delay(HandleResolveTurn, 5000);
		}
		else {
			for (i in 0...3) {
				lines[i].smugglerText.text = "";
				lines[i].customsText.text = "";
			}
			state.NextState();
			Timer.delay(advanceState, 400);	
		}
		//TODO - show value sums maybe?
			
	}
	
	//7.
	public function HandlePlayImmediateCards() {
		if (StartView.GAME_TYPE == StartView.GAME_TYPE_CUSTOMS)
		{
			//Player
			this.interactiveChildren = true;
			this.turnDone.visible = true;
			trace("player input");
		}
		else
		{
			this.enemy.react();
			state.NextState();
			Timer.delay(advanceState, 400);
		}
		//currentPosition = ResolveTurn;
		// start animation etc.. After animation currenPosition++
	}
	
	//8.
	public function HandleResolveFinal() {
		// start animation etc.. After animation currenPosition++
		//currentPosition = DealCustomsCards;
		// calculate value
		trace("FINAL");
		for (i in 0...lines.length) {
			var transport = lines[i].getTransport();
			var officer = lines[i].getOfficer();
			var mutator = lines[i].getMutator();
			
			var mutatorMultiplier:Int = mutator.mutatorMultiplier();
			// First part: lets see if there's revealed contraband
			if (transport.hasRevealedContraband()) {
				trace("revealed contraband");
				var smugglerValue = transport.calculateValue(false);
				customs.addPoints(smugglerValue*mutatorMultiplier);
			}
			else if (mutator.isAnEvent()) {
				// animation for dire?
				trace("is an event");
				var throwValue:Int= DiceThrow();
				var requiredForCustomsToWin = 0;
				if (mutator.data.type == "vihje") {
					requiredForCustomsToWin = 2;
				}
				if (mutator.data.type == "kissanristiaiset") {
					requiredForCustomsToWin = 6;
				}
				if (mutator.data.type == "pikkujoulut") {
					requiredForCustomsToWin = 2;
				}
				
				if (mutator.data.type == "virka_apu") {
					requiredForCustomsToWin = 2;
				}
				if (mutator.data.type == "haat") {
					requiredForCustomsToWin = 5;
				}
				if (mutator.data.type == "out_of_order") {
					// kumpikaan ei saa mitään.
					// Ei tartte tässä tehdä mitään.
				}
				else if (throwValue >= requiredForCustomsToWin) {
					customs.addPoints(transport.calculateValue(false));
				}
				else {
					smuggler.addPoints(transport.calculateValue(false));
				}
			}	
			else {
				// Else, have to calculate values and compare them
				var smugglerValue = transport.calculateValue(true);
				var customsValue = officer.calculateValue(true);
				var mutatorEffectOnValue = mutator.mutatorMultiplier() ;
				
				if (mutator.data.type != "lahjomaton") {
					customsValue -= transport.calculateGiftValues();
				}
				if (mutator.data.type == "lahjottu") {
					customsValue -= 1; 
				}
				if (mutator.data.type == "portti_rikki") {
					customsValue -= 1;
				}
				
				trace("smuggler vs customs");
				if ((smugglerValue+mutatorEffectOnValue) > customsValue) {
					//Smuggler wins
					Sounds.playEffect(Sounds.AUTO_LAHTEE,0,3);
					smuggler.addPoints(smugglerValue * mutatorMultiplier);
				}
				else {
					Sounds.playEffect(Sounds.JERE_PIKKUJOULUT,0,3);
					customs.addPoints(smugglerValue*mutatorMultiplier);
					//Customs wins this round
				}
			}	
		}
		state.NextState();
			Timer.delay(advanceState, 400);
	}
	
	private function handleCleanUp():Void
	{
		var c:Int = 0;
		for (l in lines)
		{
			var transport:Card = l.getTransport();
			var officer:Card = l.getOfficer();
			var mutator:Card = l.getMutator();
			//smuggler.handDeck.discardCard(
			
			
			//discard all attached cards to hand decks
			for (attached in transport.attachedCards)
			{
				smuggler.handDeck.discardCard(attached, ++c*50);
			}
			transport.attachedCards = [];
			
			for (attached in officer.attachedCards)
			{
				customs.handDeck.discardCard(attached,++c*50);
			}
			officer.attachedCards = [];
			
			smuggler.lineDeck.discardCard(transport,++c*50);
			customs.lineDeck.discardCard(officer,++c*50);
			smuggler.eventDeck.discardCard(mutator,++c*50);
			l.hasPlayerCard = false;
		}
		state.NextState();
		Timer.delay(advanceState, 450);
		if (smuggler.lineDeck.cards.length == 0) {
			//game end
			state.goToEnd();
		}
	}
	
	public function playCard(card:Card, targetLine:Line, delay:Int) {
		if (StartView.GAME_TYPE == "smuggler") {
			targetLine.setCustomsOfficer(card, delay);	
		}
		else {
			targetLine.setTransport(card,delay);
		}
	}
	
	private function DiceThrow():Int {
		
		return cast Math.floor(Math.random() * 6) + 1;
	}
}