package controls;

import controls.StateButton.State;
import createjs.soundjs.SoundInstance;
import createjs.tweenjs.Ease;
import createjs.tweenjs.Tween;
import haxe.Timer;
import pixi.core.display.Container;
import pixi.core.math.Point;
import pixi.core.math.shapes.Rectangle;
import pixi.core.sprites.Sprite;
import pixi.extras.TilingSprite;
import sounds.Sounds;
import util.Asset;

/**
 * ...
 * @author Henri Sarasvirta
 */
class StartView extends Container
{
	public static var TYPE_SELECTED:String = "typeSelected";
	public static var GAME_TYPE:String;
	
	public static var GAME_TYPE_SMUGGLER = "smuggler";
	public static var GAME_TYPE_CUSTOMS = "customs";
	
	
	
	private var bg:TilingSprite;
	
	private var logo:Sprite;
	
	private var smuggler:StateButton;
	private var customs:StateButton;
	
	private var bgs:Array<String> = ["start_0", "start_1", "start_2", "start_3"];
	private var soundloopTimer:Timer;
	
	private var lowBg:Bool = false;
	private var bgText:Sprite;
	
	public function new() 
	{
		super();
		this.initializeControls();
	}
	
	private function initializeControls():Void
	{
		this.bg = new TilingSprite( Asset.getTexture("start.png", false),100,100);
		this.bgText = Asset.getImage("tekstit.png", false);
		this.bgText.scale.x = this.bgText.scale.y = 2;
		
		this.smuggler = new StateButton();
		this.smuggler.addState("default", Asset.getImage("Rallimies.png", true));
		
		this.customs = new StateButton();
		this.customs.addState("default", Asset.getImage("Tullimies.png", true));
		
		this.smuggler.pivot = new Point(this.smuggler.width / 2, this.smuggler.height / 2);
		this.customs.pivot = new Point(this.customs.width / 2, this.customs.height / 2);

		
		this.smuggler.scale.x = this.smuggler.scale.y = 3;
		this.customs.scale.x = this.customs.scale.y = 3;
		
		
		this.interactiveChildren = true;
		
		
		this.smuggler.x = 1280/3;
		this.smuggler.y = 720/2+140;
		this.customs.x = 1280/3*2;
		this.customs.y = 720 / 2+140;
		
		
		this.logo = Asset.getImage("logo.png", true);
		this.logo.anchor.x = 0.5;
		this.logo.anchor.y = 0.5;
		this.logo.x = 1280 / 2;
		this.logo.y = 250;
		
		this.addChild(this.bg);
		this.addChild(this.bgText);
		this.addChild(this.logo);
		this.addChild(this.smuggler);
		this.addChild(this.customs);
		
		this.smuggler.addListener("click", onSmugglerClick);
		this.customs.addListener("click", onCustomsClick);
		this.smuggler.addListener("tap", onSmugglerClick);
		this.customs.addListener("tap", onCustomsClick);
		
//		this.logoTimer.run = onlogoloop;
		onlogoloop();
		
		this.bgText.y = 720;
		Tween.get(this.bgText, { loop:true } ).to( { y: -this.bgText.height - 400 }, 50000);
		
	}
	
	private var logoTimer:Timer = new Timer(3000);
	
	private function onlogoloop() {
			Tween.get(this.logo.scale, {loop:true}).to( { x : 2.0, y:2.0 }, 350, Ease.backOut )
		.to( { x : 0.7, y:0.7 }, 350, Ease.backIn )
		.to( { x : 1.5, y:1.5 }, 350, Ease.backOut )
		.to( { x : 0.7, y:0.7 }, 350, Ease.backIn );
		
		Tween.get(this.logo, {loop:true}).to( { rotation:0.3 }, 350, Ease.backOut )
		.to( { rotation:0.0 }, 350, Ease.backIn )
		.to( { rotation: -0.3 }, 350, Ease.backOut )
		.to( { rotation:0.0 }, 350, Ease.backIn );
		
		Tween.get(this.smuggler.scale,{loop:true}).wait(350,true).to( { x:3.4, y:3.4 }, 150).to({x:3, y:3},150).wait(350,true);	
		Tween.get(this.customs.scale, { loop:true } ).wait(650, true).to( { x:3.4, y:3.4 }, 150).to( { x:3, y:3 }, 150);	
	}
	
	public function prepareView():Void
	{
		this.soundloopTimer = new Timer(500);
		this.soundloopTimer.run = onTick;
		lowBg = false;
		
		this.logo.scale.x = this.logo.scale.y = 0.7;
		this.logo.rotation = 0;
		
		
		
		
	}
	
	private function onTick():Void
	{
		this.soundloopTimer.stop();
		if (bgs.length == 0)
			bgs = ["start_0", "start_1", "start_2", "start_3"];
		var sound:String = bgs[Math.floor(Math.random() * bgs.length)];
		
		var sound:SoundInstance = Sounds.playEffect("start_" + Math.floor(Math.random() * 4), 0, 0);
		Tween.get(sound).to( { volume: lowBg ? 0.2:1 }, 300);
		
		sound.addEventListener("complete", onSoundComplete);
	}
	
	private function onSoundComplete():Void
	{
		this.soundloopTimer = new Timer(1000);
		this.soundloopTimer.run = onTick;
	}
	
	private function onSmugglerClick(e:Dynamic):Void
	{
		GAME_TYPE = "smuggler";
		this.soundloopTimer.stop();
		this.lowBg = true;
		this.emit(StartView.TYPE_SELECTED);
	}
	
	private function onCustomsClick(e:Dynamic):Void
	{
		GAME_TYPE = "customs";
		this.soundloopTimer.stop();
		this.lowBg = true;
		this.emit(StartView.TYPE_SELECTED);
	}
	
	public function resize(size:Rectangle):Void
	{
		this.bgText.x = Math.round(size.width - this.bgText.width ) / 2;
		this.bg.width = size.width;
		this.bg.height = size.height;
		this.logo.x = size.width / 2;
		
		this.bgText.y = size.height;
		Tween.removeTweens(this.bgText);
		Tween.get(this.bgText, { loop:true } ).to( { y: -this.bgText.height - 200 }, 50000);

		
		this.smuggler.x = size.width/4;
		this.smuggler.y = size.height/2;
		this.customs.x = size.width/4*3;
		this.customs.y = size.height/2;
		
	}
}