package controls;
import haxe.ds.StringMap.StringMap;
import pixi.core.display.Container;
import pixi.core.display.DisplayObject;

/**
 * ...
 */
class StateButton extends Container
{
	public var state(get, set):String;
	
	private var states:StringMap<State>;
	private var _activeState:String;
	
	
	public function new()
	{
		super();
		states = new StringMap();
		this.interactive = true;
		this.defaultCursor = "pointer";
		this.buttonMode = true; 
		this.addListener("mouseover", onMouseOver);
		this.addListener("mouseout", onMouseOut);
		this.addListener("mousedown", onMouseDown);
	}
	private function onMouseOver(event:Dynamic):Void
	{
		var s:State = this.states.get(_activeState);
		s.up.visible = s.overlay;
		s.down.visible = false;
		s.disabled.visible = false;		
		s.hover.visible = true;
	}
	private function onMouseOut(event:Dynamic):Void
	{
		var s:State = this.states.get(_activeState);
		s.hover.visible = false;
		s.down.visible = false;
		s.disabled.visible = false;		
		s.up.visible = true;
	}
	private function onMouseDown(event:Dynamic):Void
	{
		var s:State = this.states.get(_activeState);
		s.up.visible = s.overlay;
		s.hover.visible = false;
		s.disabled.visible = false;		
		s.down.visible = true;
	}
	
	public function addState(name:String, up:DisplayObject, ?hover:DisplayObject, ?down:DisplayObject, ?disabled:DisplayObject, ?overlay:Bool):Void
	{
		if (_activeState == null)
		{
			_activeState = name;
		}
		var state = new State();
		state.up = up;
		state.hover = hover != null ? hover : up;
		state.down = down != null ? down : up;
		state.disabled = disabled != null ? disabled : up;
		state.name = name;
		states.set(name, state);
		
		state.hover.visible = false;
		state.down.visible = false;
		state.disabled.visible = false;
		state.up.visible = true;
		
		this.addChild(state.up);
		this.addChild(state.hover);
		this.addChild(state.down);
		this.addChild(state.disabled);
	}
	 
	private function get_state():String
	{
		return this._activeState;
	}
	
	private function set_state(value:String):String
	{
		if (value == _activeState)
			return value;
		var state:State = this.states.get(_activeState);
		var nextState:State = this.states.get(value);
		
		
		nextState.up.visible = state.up.visible;
		nextState.down.visible = state.down.visible;
		nextState.hover.visible = state.hover.visible;
		nextState.disabled.visible = state.disabled.visible;
		
		state.up.visible = false;
		state.down.visible = false;
		state.hover.visible = false;
		state.disabled.visible = false;
		
		
		this._activeState = value;
		return value;
	}
}
class State
	{
		public var overlay:Bool;
		public var hover:DisplayObject;
		public var down:DisplayObject;
		public var up:DisplayObject;
		public var disabled:DisplayObject;
		public var name:String;
		
		public function new(){}
	}