package sounds;
import createjs.soundjs.Sound;
import createjs.soundjs.SoundInstance;

import haxe.ds.StringMap.StringMap;
import haxe.Json;
import haxe.Resource;


/**
* Simple sound manager for sounds.
*/
class Sounds
{
	/**
	 * Possible sounds. Mapped to id.
	 */
	public static var CLICK:String = "click";
	public static var CARD_DRAW:String = "cardDraw";
	
	public static var BACKGROUND:String = "tulli";
	public static var BACKGROUND_DANGER:String = "tulli_danger";
	
	public static var START_0:String = "start_0";
	public static var START_1:String = "start_1";
	public static var START_2:String = "start_2";
	public static var START_3:String = "start_3";
	
	public static var AUTO_LAHTEE:String = "Auto_Lahtee";
	public static var MAUTO_LAHTEE:String = "Mauto_Lahtee";
	public static var REKKA_LAHTEE:String = "Rekka_Lahtee";
	
	public static var HANNA_HYVALTA_NAYTTAA:String = "Hanna_-_hyvalta_nayttaa";
	public static var HANNA_NEUVOSTOLIITTO:String = "Hanna_-_neuvostoliitto";
	public static var HANNA_PIKKUJOULUT:String = "Hanna_-_pikkujoulut";
	public static var HANNA_SEIS:String = "Hanna_-_seis";
	public static var HANNA_SNIFSNIF:String = "Hanna_-_snifsnif";
	public static var HANNA2_HYVALTA_NAYTTAA:String = "Hanna2_-_hyvalta_nayttaa";
	public static var HANNA2_NEUVOSTOLIITTO:String = "Hanna2_-_neuvostoliitto";
	public static var HANNA2_PIKKUJOULUT:String = "Hanna2_-_pikkujoulut";
	public static var HANNA2_SEIS:String = "Hanna2_-_seis";
	public static var HANNA2_SNIFSNIF:String = "Hanna2_-_snifsnif";
	public static var JERE_HELGA:String = "Jere_-_helga";
	public static var JERE_KAIKKI_OK:String = "Jere_-_kaikki_ok";
	public static var JERE_NEUVOSTOLIITTO:String = "Jere_-_neuvostoliitto";
	public static var JERE_PIKKUJOULUT:String = "Jere_-_pikkujoulut";
	public static var JERE_SEIS:String = "Jere_-_seis";
	public static var JERE_SNIFSNIF:String = "Jere_-_snifsnif";
	public static var JERE_VIIVA:String = "Jere_-_viiva";
	public static var JERE2_KAIKKI_HYVIN:String = "Jere2_-_kaikki_hyvin";
	public static var JERE2_KAMARILLE:String = "Jere2_-_kamarille";
	public static var JERE2_SYVA_KYYKKY:String = "Jere2_-_syva_kyykky";	
	
	public static var LUCAS_EI_TULLATTAVAA:String = "Lukas_-_Ei_tullattavaa";	
	
	/**
	* Private properties of the class.
	*/
	private static var soundMap:StringMap<SoundInstance>;
	private static var sounds:Array<String>;
	private static var initok:Bool = false;
	
	private static var _loaded:Int = 0;
	
	public static var soundsLoadedHandler:Dynamic;
	
	/**
	* Initializes the background sound.
	* @param framework A reference to the framework.
	*/
	static public function initSounds():Void
	{
		//Wrap around dynamic until externs are updated.
		
		
		soundMap = new StringMap();
		var base:String = Config.GAME_PATH + "snd/";
		Sound.alternateExtensions = ["mp3"];
		
		sounds = [
			BACKGROUND,
			BACKGROUND_DANGER,
			START_0,
			START_1,
			START_2,
			START_3,
			AUTO_LAHTEE,
			MAUTO_LAHTEE,
			REKKA_LAHTEE,
			HANNA_HYVALTA_NAYTTAA,
			HANNA_NEUVOSTOLIITTO,
			HANNA_PIKKUJOULUT,
			HANNA_SEIS,
			HANNA_SNIFSNIF,
			HANNA2_HYVALTA_NAYTTAA,
			HANNA2_NEUVOSTOLIITTO,
			HANNA2_PIKKUJOULUT,
			HANNA2_SEIS,
			HANNA2_SNIFSNIF,
			JERE_HELGA,
			JERE_KAIKKI_OK,
			JERE_NEUVOSTOLIITTO,
			JERE_PIKKUJOULUT,
			JERE_SEIS,
			JERE_SNIFSNIF,
			JERE_VIIVA,
			JERE2_KAIKKI_HYVIN,
			JERE2_KAMARILLE,
			JERE2_SYVA_KYYKKY,
			LUCAS_EI_TULLATTAVAA
		];
		
		
		var d:Dynamic = Sound;
		d.addEventListener("fileload", soundLoadHandler);
		
		//Load single sounds
		for (s in sounds)
		{
			Sound.registerSound(base + s+".ogg", s, 1);
		}
		
		//Make sure thresholding doesn't cut the music. Sounds are prebalanced so no need to do dynamic compression.
		if (Sound.activePlugin != null && Sound.activePlugin.dynamicsCompressorNode != null)
		{
			Sound.activePlugin.dynamicsCompressorNode.threshold.value = -10;
			Sound.activePlugin.dynamicsCompressorNode.ratio.value = 5;
		}
		
	}
	
	private static function soundLoadHandler():Void
	{
		_loaded++;
		if (sounds.length == _loaded)
		{
			initok = true;
			soundsLoadedHandler();
		}
	}
	
	/**
	* Plays the effect sound.
	*/
	static public function playEffect(name:String,?loops:Int, ?volume:Float, ?delay:Float):SoundInstance
	{
		if (!Sound.getMute() && initok)
		{
			if (volume == null)
				volume = 1;
			if (loops == null)
				loops = 0;
			if (delay == null)
				delay = 0;
			var si:SoundInstance = Sound.play(name, Sound.INTERRUPT_LATE, delay, 0, loops, volume);
			soundMap.set(name, si);
			return si;
		}
		return null;
	}
	
	/**
	 * Stops a sound if such exists.
	 * @param	name
	 */
	static public function stopSound(name:String):Void
	{
		if (soundMap.exists(name))
			soundMap.get(name).stop();
	}
	
}
